%--------------------------------------------------------------------------
% Total Lagrangian first order conservation law for fast solid dynamics
% Stabilised Petrov-Galerkin algorithm
% February 10, 2018
%--------------------------------------------------------------------------

%--------------------------------------------------------------------------
% clear and close all variables
%--------------------------------------------------------------------------
close all;
clc;

%--------------------------------------------------------------------------
% Set up the directory path. 
%--------------------------------------------------------------------------
if isunix()
    dirsep =  '/';    
else   
    dirsep =  '\';
end
basedir_fem = mfilename('fullpath');
basedir_fem = fileparts(basedir_fem);
basedir_fem = strrep(basedir_fem,['code' dirsep 'support'],'');
addpath(fullfile(basedir_fem));    
addpath(genpath(fullfile(basedir_fem,'code')));       
addpath((fullfile(basedir_fem,'job_folder')));   

%--------------------------------------------------------------------------
% Welcomes the user on screen.
%--------------------------------------------------------------------------
fprintf([' P R O G R A M    First order conservation laws \n Petrov Galerkin Finite element LArGe'...
         ' Strain HYperelasticity \n Program \n\n']);

%--------------------------------------------------------------------------
% Reads in the input file name.
%--------------------------------------------------------------------------
PRO.inputfile_name = input([' Enter the data file name   : ' ' \n'],'s');
    
%----------------------------------------------------------------------
% Define the job folder location.
%----------------------------------------------------------------------
PRO.job_folder = fullfile(basedir_fem,['job_folder\'...
    PRO.inputfile_name(1:end-4)]);

%----------------------------------------------------------------------
% Check if this folder has been defined by the user or not.
%----------------------------------------------------------------------
if ~exist(PRO.job_folder,'dir')
    error(['The folder ' PRO.job_folder ' has not been ' ...
        'created. Please define the folder and the input file (not '...
        'including the extension) identical'])
end
cd(PRO.job_folder)
    
%--------------------------------------------------------------------------
% Read user-defined control file
%--------------------------------------------------------------------------
disp('*********************************************************************');
disp('Read Control.txt');
disp('*********************************************************************');
[CON,FUNC,DAT]                       = Read_Control_File;

%--------------------------------------------------------------------------
% Read "geometry and initial condition"
%--------------------------------------------------------------------------
disp('*********************************************************************');
disp('Read Geometry and Initial conditions');
disp('*********************************************************************');
[GEO,DAT,FUNC]                       = Read_Geometry_File(FUNC,CON,DAT,PRO);

%--------------------------------------------------------------------------
% Initialisation
%--------------------------------------------------------------------------
disp('*********************************************************************');
disp('Initialisation');
disp('*********************************************************************');
[FUNC,CON,DAT,GEO,UNK,COMP]          = Initialisation(FUNC,CON,DAT,GEO);

%----------------------------------------------------------------------
% Run through Command Window
%----------------------------------------------------------------------
Run_Jobs_Batch_Background(CON,DAT,GEO,FUNC,UNK,COMP,PRO)
    