%--------------------------------------------------------------------------
% Post-Processing subroutine
% Convert results to the format of PARAVIEW
%--------------------------------------------------------------------------
close all; clear all; clc;

%--------------------------------------------------------------------------
% User input
%--------------------------------------------------------------------------
TOT   = 10;
INT   = 1;

%--------------------------------------------------------------------------
% Set up the directory path. 
%--------------------------------------------------------------------------
if isunix()
    dirsep =  '/';    
else   
    dirsep =  '\';
end
basedir_fem = mfilename('fullpath');
basedir_fem = fileparts(basedir_fem);
basedir_fem = strrep(basedir_fem,['code' dirsep 'support'],'');
addpath(fullfile(basedir_fem));    
addpath(genpath(fullfile(basedir_fem,'code')));       
addpath((fullfile(basedir_fem,'job_folder'))); 

%--------------------------------------------------------------------------
% Reads in the input file name.
%--------------------------------------------------------------------------
inputfile_name = input([' Enter the folder name   : ' ' \n'],'s');
    
%----------------------------------------------------------------------
% Define the job folder location.
%----------------------------------------------------------------------
job_folder = fullfile(basedir_fem,['job_folder\'...
    inputfile_name]);

%----------------------------------------------------------------------
% Check if this folder has been defined by the user or not.
%----------------------------------------------------------------------
if ~exist(job_folder,'dir')
    error(['The folder ' job_folder ' has not been ' ...
        'created. Please define the folder and the input file (not '...
        'including the extension) identical'])
end

%--------------------------------------------------------------------------
% Change directory
%--------------------------------------------------------------------------
cd(job_folder)

%--------------------------------------------------------------------------
% Load files
%--------------------------------------------------------------------------
load('Mesh.mat')

%--------------------------------------------------------------------------
% Time 
%--------------------------------------------------------------------------
outputfile1                              = sprintf('TimeSeries.pvd');
outputfid1                               = fopen(outputfile1,'w');
fprintf(outputfid1,'<?xml version="1.0"?>\n');
fprintf(outputfid1,'<VTKFile type="Collection" version="0.1" byte_order="LittleEndian">\n');
fprintf(outputfid1,'<Collection>\n');

%--------------------------------------------------------------------------
% Loop over results
%--------------------------------------------------------------------------
for i = 1:INT:TOT/CON.OutInc
    
    %----------------------------------------------------------------------
    % Load file
    %----------------------------------------------------------------------
    s = sprintf('snapshot%010d.mat',i*CON.OutInc);
    load(s);
    s = sprintf('PGFEM_Paraview%010d.vtu',i*CON.OutInc);
    outputfid2 = fopen(s,'w');
   
    %----------------------------------------------------------------------
    % Get variables
    %----------------------------------------------------------------------
    Time     = OUT.Time;
    Pressure = OUT.UNK_Pressure;
    LM       = OUT.UNK_p;
    F        = OUT.UNK_F;
    Entropy  = OUT.UNK_Eta;
%     Theta    = OUT.UNK_Theta;
    X        = OUT.UNK_X;
    Velocity = LM ./ DAT.Den;
    
    %----------------------------------------------------------------------
    % Print time
    %----------------------------------------------------------------------
    fprintf(outputfid1,'    <DataSet timestep="%010d " group="" part="0" file="PGFEM_Paraview%010d.vtu"/>\n',Time,i*CON.OutInc);

    %----------------------------------------------------------------------
    % Generate output files for PARAVIEW
    %----------------------------------------------------------------------
    % For VTP file
    fprintf(outputfid2,'<?xml version="1.0"?>\n');
    fprintf(outputfid2,'<VTKFile type="UnstructuredGrid" version="0.1" byte_order="LittleEndian">\n');
    fprintf(outputfid2,'<UnstructuredGrid>\n');
    fprintf(outputfid2,'<Piece NumberOfPoints="%d" NumberOfCells="%d">\n',GEO.Nnode, GEO.Nelem);
    
    %----------------------------------------------------------------------
    % Geometry
    %----------------------------------------------------------------------
    fprintf(outputfid2,'<Points>\n');
    fprintf(outputfid2,'<DataArray type="Float32" NumberOfComponents="3" format="ascii">\n');    
    for ip=1:GEO.Nnode
        fprintf(outputfid2,' %f ' ,X(1,ip));
        fprintf(outputfid2,' %f\t',X(2,ip));
        fprintf(outputfid2,' %f\n',X(3,ip));
    end
    fprintf(outputfid2,'</DataArray>\n');
    fprintf(outputfid2,'</Points>\n');    
    
    %----------------------------------------------------------------------
    % Connectivity
    %----------------------------------------------------------------------
    fprintf(outputfid2,'<Cells>\n');
    fprintf(outputfid2,'<DataArray type="Int32" Name="connectivity" format="ascii">\n');    
    for ie = 1:GEO.Nelem
        fprintf(outputfid2,'%d %d %d %d \n',GEO.T(:,ie)-1);
    end
    fprintf(outputfid2,'</DataArray>\n');

    %----------------------------------------------------------------------
    % Offsets
    %----------------------------------------------------------------------
    fprintf(outputfid2,'<DataArray type="Int32" Name="offsets" format="ascii">\n');    
    Offset = GEO.NodeE;
    for ie = 1:GEO.Nelem
        fprintf(outputfid2,'%d \n',Offset*ie);
    end
    fprintf(outputfid2,'</DataArray>\n'); 
    
    %----------------------------------------------------------------------
    % Cell types
    %----------------------------------------------------------------------
    fprintf(outputfid2,'<DataArray type="UInt8" Name="types" format="ascii">\n');    
    Cell_type = zeros(1,GEO.Nelem);
    Cell_type(:) = 10;
    fprintf(outputfid2,'%d ', Cell_type);
    fprintf(outputfid2,'</DataArray>\n');
    fprintf(outputfid2,'</Cells>\n');        
    
    %----------------------------------------------------------------------
    % Pressure
    %----------------------------------------------------------------------
    fprintf(outputfid2,'<PointData Scalars="Pressure, Entropy, Temperature" Vectors="Velocity">\n');
    fprintf(outputfid2,'<DataArray type="Float32" Name="Pressure (Pa)" format="ascii">\n');
    fprintf(outputfid2,'%f\n',Pressure);
    fprintf(outputfid2,'</DataArray>\n');

    %----------------------------------------------------------------------
    % Entropy
    %----------------------------------------------------------------------
    fprintf(outputfid2,'<DataArray type="Float32" Name="Entropy" format="ascii">\n');
    fprintf(outputfid2,'%f\n',Entropy);
    fprintf(outputfid2,'</DataArray>\n');
    
%     %----------------------------------------------------------------------
%     % Temperature
%     %----------------------------------------------------------------------
%     fprintf(outputfid2,'<DataArray type="Float32" Name="Temperature" format="ascii">\n');
%     fprintf(outputfid2,'%f\n',Theta);
%     fprintf(outputfid2,'</DataArray>\n');

    %----------------------------------------------------------------------
    % Velocity vector
    %----------------------------------------------------------------------
    fprintf(outputfid2,'<DataArray type="Float32" Name="Velocity (m/s)" NumberOfComponents="3" format="ascii">\n');
    for ip=1:GEO.Nnode
        fprintf(outputfid2,' %f ' ,Velocity(1,ip));
        fprintf(outputfid2,' %f\t',Velocity(2,ip));
        fprintf(outputfid2,' %f\n',Velocity(3,ip));
    end
    fprintf(outputfid2,'</DataArray>\n');
    fprintf(outputfid2,'</PointData>\n');
        
    fprintf(outputfid2,'</Piece>\n');
    fprintf(outputfid2,'</UnstructuredGrid>\n');
    fprintf(outputfid2,'</VTKFile>\n');
    fclose(outputfid2);
    
end

%--------------------------------------------------------------------------
% Close file
%--------------------------------------------------------------------------
fprintf(outputfid1,'</Collection>\n');
fprintf(outputfid1,'</VTKFile>\n');
fclose(outputfid1);
