%--------------------------------------------------------------------------
% Impose strong boundary conditions on linear momentum
%--------------------------------------------------------------------------
function [UNK] = Apply_strong_boundary_conditions(UNK,GEO)

%--------------------------------------------------------------------------
% Extraction
%--------------------------------------------------------------------------
GEO_Nface     = GEO.Nface;
GEO_T_B       = GEO.T_B;
GEO_N_B       = GEO.N_B;
GEO_Bcode     = GEO.Bcode;
UNK_p         = UNK.p;

%--------------------------------------------------------------------------
% Loop over the boundary faces
%--------------------------------------------------------------------------
for iface = 1:GEO_Nface
    
    %----------------------------------------------------------------------
    % Extract boundary connectivity
    %----------------------------------------------------------------------
    lnodsB        = GEO_T_B(:,iface);
    
    %----------------------------------------------------------------------
    % Extract area and its outward normal vector
    %----------------------------------------------------------------------
    Normal        = GEO_N_B(:,iface);
    
    %----------------------------------------------------------------------
    % Type of boundary conditions
    %----------------------------------------------------------------------
    Type_BC       = GEO_Bcode(iface);
    switch Type_BC
        
        case(2)
        %------------------------------------------------------------------
        % Fixed wall boundary condition
        %------------------------------------------------------------------
        UNK_p(:,lnodsB)      = 0;
            
        case(3)
            
        %------------------------------------------------------------------
        % Symmetric boundary condition
        %------------------------------------------------------------------
        UNK_p(:,lnodsB)      = (eye(3) - Normal * Normal') * UNK_p(:,lnodsB);
       
        case(4)
        %------------------------------------------------------------------
        % Non-symmetric boundary condition
        %------------------------------------------------------------------
        UNK_p(:,lnodsB)      = (Normal * Normal') * UNK_p(:,lnodsB);
              
        case(6)
            
        %------------------------------------------------------------------
        % Traction + roller
        %------------------------------------------------------------------
        UNK_p(:,lnodsB)      = (eye(3) - Normal * Normal') * UNK_p(:,lnodsB);
        
    end
    
end

%--------------------------------------------------------------------------
% Assign linear momentum
%--------------------------------------------------------------------------
UNK.p                        = UNK_p;

end