%--------------------------------------------------------------------------
% Imposition of boundary conditions
%--------------------------------------------------------------------------
function [TractionOUT] = Type_of_boundary(TractionIN,Normal,Area,F,Bcode,ExtPres,TB,TB_Neg,time)

%--------------------------------------------------------------------------
% Define spatial outward unit normal vector
%--------------------------------------------------------------------------
Inv_F        = F^(-1);
Inv_FT       = Inv_F';
tmp          = Inv_FT * Normal;
tmp_norm     = sqrt(tmp' * tmp);
normal       = tmp / tmp_norm;

%--------------------------------------------------------------------------
% Different types of boundaries
% 1 - Free boundary condition
% 2 - Fixed wall boundary condition
% 3 - Symmetric boundary condition
% 4 - Non-Symmetric boundary condition
% 5 - Pressure boundary 
% 6 - Traction + roller boundaries
%--------------------------------------------------------------------------
switch Bcode
    
    case (1)
        
        %------------------------------------------------------------------
        % Free  boundary condition
        %------------------------------------------------------------------
        TractionOUT       = zeros(3,1);
     
    case (2)
        
        %------------------------------------------------------------------
        % Fixed wall boundary condition
        %------------------------------------------------------------------
        TractionOUT       = TractionIN;
        
    case(3)
        
        %------------------------------------------------------------------
        % Symmetric boundary condition
        %------------------------------------------------------------------
        TractionOUT       = (Normal * Normal') * TractionIN;
       
    case(4)
        
        %------------------------------------------------------------------
        % Non-symmetric boundary condition
        %------------------------------------------------------------------
        TractionOUT       = (eye(3) - Normal * Normal') * TractionIN;
     
    case(5)
        
        %------------------------------------------------------------------
        % Internal pressure
        %------------------------------------------------------------------
        TractionOUT       =  ExtPres * normal;
        
    case(6)
        
        %------------------------------------------------------------------
        % Traction  + roller
        %------------------------------------------------------------------
        TractionOUT       =  TB;
          
    case(7)
        
        %------------------------------------------------------------------
        % Traction  
        %------------------------------------------------------------------
        if (time < 2.5)
            TractionOUT       = TB * time;
        elseif (time < 5)
            TractionOUT       = TB * (5-time);
        else
            TractionOUT       = [0;0;0];
        end
        
    case(8)
        
        %------------------------------------------------------------------
        % Traction 
        %------------------------------------------------------------------
        if (time < 2.5)
            TractionOUT       = TB_Neg * time;
        elseif (time < 5)
            TractionOUT       = TB_Neg * (5-time);
        else
            TractionOUT       = [0;0;0];
        end          
end

end