function [COMP_mass,Grad0,J0gp] = Lumped_mass_matrix(GEO)

%--------------------------------------------------------------------------
% Extraction
%--------------------------------------------------------------------------
GEO_NodeE                              = GEO.NodeE;
GEO_Nnode                              = GEO.Nnode;
GEO_T                                  = GEO.T;
GEO_X                                  = GEO.X;
GEO_Nelem                              = GEO.Nelem;

%--------------------------------------------------------------------------
% 3D tetrahedral mesh 
% Only consider 1 quadrature point
%--------------------------------------------------------------------------
ISO_Gradxi0                            = zeros(GEO_NodeE,3);       % Gradient of shape functions in parent domain
ISO_Gradxi0(:,1,1)                     = [-1  1  0  0]';
ISO_Gradxi0(:,2,1)                     = [-1  0  1  0]';
ISO_Gradxi0(:,3,1)                     = [-1  0  0  1]';
ISO_Wgp                                = 1/6;                      % Weight of quadrature point

%--------------------------------------------------------------------------
% Initiation
%--------------------------------------------------------------------------
COMP_mass                         = zeros(GEO_Nnode,1);
Grad0                             = zeros(GEO_NodeE,3,GEO_Nelem);
J0gp                              = zeros(1,GEO_Nelem);

%--------------------------------------------------------------------------
% Compute mass
%--------------------------------------------------------------------------
for ielem = 1:GEO_Nelem
    
    %----------------------------------------------------------------------
    % Extract local coordinates per element
    %----------------------------------------------------------------------
    Xelem                         = GEO_X(:,GEO_T(:,ielem));
    
    %----------------------------------------------------------------------
    % Cartesian derivative of the corresponding shape functions
    %----------------------------------------------------------------------
    [Grad0,J0gp]                  = Cartelementinfo(GEO_NodeE,ISO_Gradxi0,ISO_Wgp,Xelem,ielem,Grad0,J0gp);

    %----------------------------------------------------------------------
    % Lumped Mass
    %----------------------------------------------------------------------
    COMP_mass(GEO_T(:,ielem))     =  COMP_mass(GEO_T(:,ielem)) + J0gp(ielem)/GEO_NodeE;
    
end

end