%--------------------------------------------------------------------------
% Global angular momentum projection through least square minisation procedure
%--------------------------------------------------------------------------
function [RHS] = Angular_Momentum_Projection(RHS,Mass,X,Nnode)

%--------------------------------------------------------------------------
% Determine the rate of linear momentum
%--------------------------------------------------------------------------
pdot                            = zeros(3,Nnode);
pdot(1,:)                       = RHS(1,:) ./ Mass';
pdot(2,:)                       = RHS(2,:) ./ Mass';
pdot(3,:)                       = RHS(3,:) ./ Mass';

%--------------------------------------------------------------------------
% Initiation
%--------------------------------------------------------------------------
K_XX                     = zeros(3,3);
K_XY                     = zeros(3,3);
K_YX                     = K_XY;
K_YY                     = 0;
RX                       = zeros(3,1);
RY                       = zeros(3,1);

%--------------------------------------------------------------------------
% Stiffness-like matrices 
%--------------------------------------------------------------------------
Par                       = X(1,:) .* X(1,:) + X(2,:)  .* X(2,:) + X(3,:) .* X(3,:);
K_XX(1,1)                 = (Par - X(1,:) .* X(1,:))   * Mass;
K_XX(1,2)                 = -(X(1,:) .* X(2,:))        * Mass;
K_XX(1,3)                 = -(X(1,:) .* X(3,:))        * Mass;
K_XX(2,2)                 = (Par - X(2,:) .* X(2,:))   * Mass;
K_XX(2,3)                 = -(X(2,:) .* X(3,:))        * Mass;
K_XX(3,3)                 = (Par - X(3,:) .* X(3,:))   * Mass;
K_XX(2,1)                 = K_XX(1,2);
K_XX(3,1)                 = K_XX(1,3);
K_XX(3,2)                 = K_XX(2,3);
%
K_XY(1,1)                 = 0;
K_XY(1,2)                 = -X(3,:) * Mass;
K_XY(1,3)                 =  X(2,:) * Mass;
K_XY(2,1)                 =  X(3,:) * Mass;
K_XY(2,2)                 = 0;
K_XY(2,3)                 = -X(1,:) * Mass;
K_XY(3,1)                 = -X(2,:) * Mass;
K_XY(3,2)                 =  X(1,:) * Mass;
K_XY(3,3)                 = 0;
K_YX                      = K_XY;
K_YY                      = - sum(Mass);

%--------------------------------------------------------------------------
% Right-hand-side of system of equations
%--------------------------------------------------------------------------
RX(1)                     = (pdot(2,:) .* X(3,:) - pdot(3,:) .* X(2,:) ) * Mass;
RX(2)                     = (pdot(3,:) .* X(1,:) - pdot(1,:) .* X(3,:) ) * Mass;
RX(3)                     = (pdot(1,:) .* X(2,:) - pdot(2,:) .* X(1,:) ) * Mass;

%--------------------------------------------------------------------------
% Find Lagrange multiplier
%--------------------------------------------------------------------------
K                         = K_XX - K_XY * K_YX / K_YY;
R                         = RX;
lambda                    = K \ R;
beta                      = - (K_YX * lambda) / K_YY;

%--------------------------------------------------------------------------
% Angular momentum projection procedure
%--------------------------------------------------------------------------
Beta_matrix      = zeros(size(RHS));
Beta_matrix(1,:) = beta(1);
Beta_matrix(2,:) = beta(2);
Beta_matrix(3,:) = beta(3);
%
pdot(1,:)       = pdot(1,:) + (lambda(2) * X(3,:) - lambda(3) * X(2,:)) + Beta_matrix(1,:);
pdot(2,:)       = pdot(2,:) + (lambda(3) * X(1,:) - lambda(1) * X(3,:)) + Beta_matrix(2,:);
pdot(3,:)       = pdot(3,:) + (lambda(1) * X(2,:) - lambda(2) * X(1,:)) + Beta_matrix(3,:);

RHS(1,:)        = pdot(1,:) .* Mass';
RHS(2,:)        = pdot(2,:) .* Mass';
RHS(3,:)        = pdot(3,:) .* Mass';

end
