function [Rp] = Boundary_Contribution(FUNC,UNK,GEO,DAT,COMP,Rp)

%--------------------------------------------------------------------------
% Extraction
%--------------------------------------------------------------------------
GEO_Nface          = GEO.Nface;
GEO_T_B            = GEO.T_B;
GEO_A_B            = GEO.A_B;
GEO_N_B            = GEO.N_B;
GEO_NodeF          = GEO.NodeF;
GEO_Bcode          = GEO.Bcode;
DAT_Temp_R         = DAT.Temp_R;
DAT_ExtPres        = DAT.ExtPres;
DAT_TB             = DAT.TB;
DAT_TB_Neg         = DAT.TB_Neg;
UNK_p              = UNK.p;
UNK_F              = UNK.F;
UNK_Eta            = UNK.Eta;
FUNC_Piola_kirchhoff = FUNC.Piola_kirchhoff;
COMP_time          = COMP.time;

%--------------------------------------------------------------------------
% Loop over the boundary faces
%--------------------------------------------------------------------------
for iface = 1:GEO_Nface
        
    %----------------------------------------------------------------------
    % Extract area and its outward normal vector
    %----------------------------------------------------------------------
    Area          = GEO_A_B(iface);
    Normal        = GEO_N_B(:,iface);
    
    %----------------------------------------------------------------------
    % Interpolated variables @ Gauss quadrature point
    %----------------------------------------------------------------------
    p_gp          = zeros(3,1);
    F_gp          = zeros(3,3);
    Eta_gp        = 0;
    ThetaR_gp     = 0;
    for innode = 1:GEO_NodeF
        
        %------------------------------------------------------------------
        % Boundary node ID
        %------------------------------------------------------------------
        Node_ID   = GEO_T_B(innode,iface);
        
        %------------------------------------------------------------------
        % Extract p and {F, H, J}
        %------------------------------------------------------------------
        p         = UNK_p(:,Node_ID);
        F         = UNK_F(:,:,Node_ID);
        Eta       = UNK_Eta(Node_ID);
        ThetaR    = DAT_Temp_R(Node_ID);
        
        %------------------------------------------------------------------
        % Variables @ Gauss quadrature point
        %------------------------------------------------------------------
        p_gp      = p_gp + p / GEO_NodeF;
        F_gp      = F_gp + F / GEO_NodeF;
        Eta_gp    = Eta_gp + Eta / GEO_NodeF;
        ThetaR_gp = ThetaR_gp + ThetaR / GEO_NodeF;
    end
    
    %----------------------------------------------------------------------
    % Area and volume maps
    %----------------------------------------------------------------------
    H_gp          = 0.5 * tensor_cross_product(F_gp,F_gp);
    J_gp          = det(F_gp);
    
    %----------------------------------------------------------------------
    % First Piola Kirchhoff stress tensor
    %----------------------------------------------------------------------
    [P]           = FUNC_Piola_kirchhoff(COMP,DAT,F_gp,H_gp,J_gp,F_gp,H_gp,J_gp,Eta_gp,ThetaR_gp);
    Traction      = P * Normal;
    
    %----------------------------------------------------------------------
    % Imposition of boundary conditions
    %----------------------------------------------------------------------
    [Traction]   = Type_of_boundary(Traction,Normal,Area,F_gp,GEO_Bcode(iface),DAT_ExtPres,DAT_TB,DAT_TB_Neg,COMP_time);
    
    %----------------------------------------------------------------------
    % Loop over the number of nodes per boundary face
    %----------------------------------------------------------------------
    for in = 1:GEO_NodeF
        
        %------------------------------------------------------------------
        % Boundary node ID
        %------------------------------------------------------------------
        Node_ID   = GEO_T_B(in,iface);
        
        %------------------------------------------------------------------
        % Add nodal contributions to the right-hand sides of linear
        % momentum and fibre map evolutions
        %------------------------------------------------------------------
        Rp(:,Node_ID)      = Rp(:,Node_ID)   + Traction * Area / GEO_NodeF;

    end
    
end


end