%--------------------------------------------------------------------------
% Local angular momentum projection
%--------------------------------------------------------------------------
function [P] = Local_Angular_Momentum_Projection(Fx,P)

%--------------------------------------------------------------------------
% Define left Cauchy Green tensor
%--------------------------------------------------------------------------
b                               = Fx * Fx';

%--------------------------------------------------------------------------
% Trace of b
%--------------------------------------------------------------------------
trb                             = b(1,1) + b(2,2) + b(3,3);

%--------------------------------------------------------------------------
%          T
% Define PF
%--------------------------------------------------------------------------
PFT                             = P * Fx';

%--------------------------------------------------------------------------
% Define left and right hand sides matrices
%--------------------------------------------------------------------------
LHS                             = trb * eye(3) - b;
RHS                             = zeros(3,1);
RHS(1)                          = PFT(2,3)-PFT(3,2);
RHS(2)                          = PFT(3,1)-PFT(1,3);
RHS(3)                          = PFT(1,2)-PFT(2,1);

%--------------------------------------------------------------------------
% Evaluation of Lagrange Multiplier
%--------------------------------------------------------------------------
Lambda                          = LHS\RHS;

%--------------------------------------------------------------------------
% Projection of first Piola Kirchhoff stress tensor
%--------------------------------------------------------------------------
Correction                      = zeros(3,3);
Correction(:,1)                 = cross(Lambda,Fx(:,1));
Correction(:,2)                 = cross(Lambda,Fx(:,2));
Correction(:,3)                 = cross(Lambda,Fx(:,3));
P                               = P + Correction;

end