function [RF,REta] = RHS_F_and_Eta(UNK,GEO,DAT,COMP)

%--------------------------------------------------------------------------
% Extraction
%--------------------------------------------------------------------------
GEO_Nnode         = GEO.Nnode;
GEO_NodeE         = GEO.NodeE;
GEO_Nelem         = GEO.Nelem;
GEO_T             = GEO.T;
UNK_p             = UNK.p;
UNK_F             = UNK.F;
UNK_Eta           = UNK.Eta;
DAT_Temp_R        = DAT.Temp_R;
DAT_MG_gamma      = DAT.MG_gamma;
DAT_MG_q          = DAT.MG_q;
DAT_Den           = DAT.Den;
DAT_CF            = DAT.CF;
DAT_Conduct       = DAT.Conduct;
DAT_ISO_J0gp      = DAT.ISO_J0gp;
DAT_ISO_Grad0     = DAT.ISO_Grad0;
COMP_irho         = COMP.irho;

%--------------------------------------------------------------------------
% Initiation
%--------------------------------------------------------------------------
RF                = zeros(3,3,GEO_Nnode);
REta              = zeros(GEO_Nnode,1);

%--------------------------------------------------------------------------
% Loop over the element
%--------------------------------------------------------------------------
for ielem = 1:GEO_Nelem
    
    
    %----------------------------------------------------------------------
    % Initation
    %----------------------------------------------------------------------
    F_gp                                      = [0 0 0; 0 0 0; 0 0 0];
    Grad_v_gp                                 = [0 0 0; 0 0 0; 0 0 0];
    Theta_gp                                  = 0;
    Grad_Theta_gp                             = [0;0;0];
    
    %----------------------------------------------------------------------
    % Elemental current volume
    %----------------------------------------------------------------------
    J0gp                                      = DAT_ISO_J0gp(ielem);
    
    %----------------------------------------------------------------------
    % Interpolated variables @ Gauss quadrature points
    %----------------------------------------------------------------------
    for in = 1:GEO_NodeE
        
        %------------------------------------------------------------------
        % Node id
        %------------------------------------------------------------------
        Node_ID                     = GEO_T(in,ielem);
        
        %------------------------------------------------------------------
        % Material gradient
        %------------------------------------------------------------------
        Grad0                       = DAT_ISO_Grad0(in,:,ielem);
   
        %------------------------------------------------------------------
        % Extract conservation variables
        %------------------------------------------------------------------
        p                           = UNK_p(:,Node_ID);
        F                           = UNK_F(:,:,Node_ID);        
        Eta                         = UNK_Eta(Node_ID);
        
        %------------------------------------------------------------------
        % Vvolume map
        %------------------------------------------------------------------
        J                           = det(F);
        
        %------------------------------------------------------------------
        % Thermal variables
        %------------------------------------------------------------------
        Theta0                      = DAT_Temp_R(Node_ID) * exp(-DAT_MG_gamma*(J^(DAT_MG_q)-1)/DAT_MG_q);
        Theta                       = Theta0 * exp(Eta/DAT_Den/DAT_CF);
          
        %------------------------------------------------------------------
        % Variables at Gauss quadrature point
        %------------------------------------------------------------------
        F_gp                        = F_gp           + F / GEO_NodeE;
        Grad_v_gp                   = Grad_v_gp      + COMP_irho * p * Grad0;
        Theta_gp                    = Theta_gp       + Theta / GEO_NodeE;
        Grad_Theta_gp               = Grad_Theta_gp  + Theta * Grad0';
        
    end

    %----------------------------------------------------------------------
    % Heat flux
    %----------------------------------------------------------------------
    H_gp                            = 0.5 * tensor_cross_product(F_gp,F_gp);
    J_gp                            = det(F_gp);
    Q_gp                            = - DAT_Conduct * H_gp' * H_gp/J_gp * Grad_Theta_gp;

    %----------------------------------------------------------------------
    % Loop over the number of nodes within an element
    %----------------------------------------------------------------------
    for in = 1:GEO_NodeE
        
        %------------------------------------------------------------------
        % Node id
        %------------------------------------------------------------------
        Node_ID                     = GEO_T(in,ielem);
        
        %------------------------------------------------------------------
        % Material gradient
        %------------------------------------------------------------------
        Grad0                       = DAT_ISO_Grad0(in,:,ielem);

        %------------------------------------------------------------------
        % Nodal contribution for fibre map and entropy variable
        %------------------------------------------------------------------
        REta(Node_ID)      = REta(Node_ID)      + Grad0 * Q_gp / Theta_gp * J0gp - Grad_Theta_gp' * Q_gp / Theta_gp / Theta_gp / GEO_NodeE * J0gp;
    
        %------------------------------------------------------------------
        % Nodal contribution for fibre map and entropy equation
        %------------------------------------------------------------------
        RF(:,:,Node_ID)    = RF(:,:,Node_ID)    + Grad_v_gp    / GEO_NodeE * J0gp;
        
    end
    
end

end