function [RF,RH,RJ,REta] = RHS_F_and_H_and_J(UNK,GEO,DAT,COMP)

%--------------------------------------------------------------------------
% Extraction
%--------------------------------------------------------------------------
GEO_Nnode         = GEO.Nnode;
GEO_NodeE         = GEO.NodeE;
GEO_Nelem         = GEO.Nelem;
GEO_T             = GEO.T;
UNK_p             = UNK.p;
UNK_F             = UNK.F;
UNK_H             = UNK.H;
UNK_J             = UNK.J;
UNK_Eta           = UNK.Eta;
DAT_Temp_R        = DAT.Temp_R;
DAT_MG_gamma      = DAT.MG_gamma;
DAT_MG_q          = DAT.MG_q;
DAT_Den           = DAT.Den;
DAT_CF            = DAT.CF;
DAT_Conduct       = DAT.Conduct;
DAT_ISO_J0gp      = DAT.ISO_J0gp;
DAT_ISO_Grad0     = DAT.ISO_Grad0;
COMP_irho         = COMP.irho;

%--------------------------------------------------------------------------
% Initiation
%--------------------------------------------------------------------------
RF                = zeros(3,3,GEO_Nnode);
RH                = zeros(3,3,GEO_Nnode);
RJ                = zeros(GEO_Nnode,1);
REta              = zeros(GEO_Nnode,1);

%--------------------------------------------------------------------------
% Loop over the element
%--------------------------------------------------------------------------
for ielem = 1:GEO_Nelem
    
    
    %----------------------------------------------------------------------
    % Initation
    %----------------------------------------------------------------------
    J_gp                                      = 0;
    F_gp                                      = [0 0 0; 0 0 0; 0 0 0];
    H_gp                                      = [0 0 0; 0 0 0; 0 0 0]; 
    Grad_v_gp                                 = [0 0 0; 0 0 0; 0 0 0];
    Theta_gp                                  = 0;
    Grad_Theta_gp                             = [0;0;0];
    
    %----------------------------------------------------------------------
    % Elemental current volume
    %----------------------------------------------------------------------
    J0gp                                      = DAT_ISO_J0gp(ielem);
    
    %----------------------------------------------------------------------
    % Interpolated variables @ Gauss quadrature points
    %----------------------------------------------------------------------
    for in = 1:GEO_NodeE
        
        %------------------------------------------------------------------
        % Node id
        %------------------------------------------------------------------
        Node_ID                     = GEO_T(in,ielem);
        
        %------------------------------------------------------------------
        % Material gradient
        %------------------------------------------------------------------
%         Grad0                       = DAT_ISO_Grad0(in,:,ielem);
        Grad0                       = [0 0 0];
        Grad0(1)                    = DAT_ISO_Grad0(in,1,ielem);
        Grad0(2)                    = DAT_ISO_Grad0(in,2,ielem);
        Grad0(3)                    = DAT_ISO_Grad0(in,3,ielem);
        
        %------------------------------------------------------------------
        % Extract conservation variables
        %------------------------------------------------------------------
        p                           = UNK_p(:,Node_ID);
%         F                           = UNK_F(:,:,Node_ID);
%         H                           = UNK_H(:,:,Node_ID);
        J                           = UNK_J(Node_ID);
        Eta                         = UNK_Eta(Node_ID);
        
        %------------------------------------------------------------------
        % Vectorisation
        %------------------------------------------------------------------
        F                           = [0 0 0; 0 0 0; 0 0 0];
        F(1,1)                      = UNK_F(1,1,Node_ID);
        F(1,2)                      = UNK_F(1,2,Node_ID);
        F(1,3)                      = UNK_F(1,3,Node_ID);        
        F(2,1)                      = UNK_F(2,1,Node_ID);
        F(2,2)                      = UNK_F(2,2,Node_ID);
        F(2,3)                      = UNK_F(2,3,Node_ID);   
        F(3,1)                      = UNK_F(3,1,Node_ID);
        F(3,2)                      = UNK_F(3,2,Node_ID);
        F(3,3)                      = UNK_F(3,3,Node_ID);   
        
        H                           = [0 0 0; 0 0 0; 0 0 0];
        H(1,1)                      = UNK_H(1,1,Node_ID);
        H(1,2)                      = UNK_H(1,2,Node_ID);
        H(1,3)                      = UNK_H(1,3,Node_ID);        
        H(2,1)                      = UNK_H(2,1,Node_ID);
        H(2,2)                      = UNK_H(2,2,Node_ID);
        H(2,3)                      = UNK_H(2,3,Node_ID);   
        H(3,1)                      = UNK_H(3,1,Node_ID);
        H(3,2)                      = UNK_H(3,2,Node_ID);
        H(3,3)                      = UNK_H(3,3,Node_ID);   
        
        %------------------------------------------------------------------
        % Thermal variables
        %------------------------------------------------------------------
        Theta0                      = DAT_Temp_R(Node_ID) * exp(-DAT_MG_gamma*(J^(DAT_MG_q)-1)/DAT_MG_q);
        Theta                       = Theta0 * exp(Eta/DAT_Den/DAT_CF);
          
        %------------------------------------------------------------------
        % Variables at Gauss quadrature point
        %------------------------------------------------------------------
        J_gp                        = J_gp           + J / GEO_NodeE;
        F_gp                        = F_gp           + F / GEO_NodeE;
        H_gp                        = H_gp           + H / GEO_NodeE;
%         Grad_v_gp                   = Grad_v_gp      + COMP_irho * p * Grad0;
        Theta_gp                    = Theta_gp       + Theta / GEO_NodeE;
%         Grad_Theta_gp               = Grad_Theta_gp  + Theta * Grad0';
        
        %------------------------------------------------------------------
        % Vectorisation
        %------------------------------------------------------------------
        Grad_v_gp(1,1)              = Grad_v_gp(1,1) + COMP_irho * p(1) * Grad0(1);
        Grad_v_gp(1,2)              = Grad_v_gp(1,2) + COMP_irho * p(1) * Grad0(2);        
        Grad_v_gp(1,3)              = Grad_v_gp(1,3) + COMP_irho * p(1) * Grad0(3);     
        Grad_v_gp(2,1)              = Grad_v_gp(2,1) + COMP_irho * p(2) * Grad0(1);
        Grad_v_gp(2,2)              = Grad_v_gp(2,2) + COMP_irho * p(2) * Grad0(2);        
        Grad_v_gp(2,3)              = Grad_v_gp(2,3) + COMP_irho * p(2) * Grad0(3);     
        Grad_v_gp(3,1)              = Grad_v_gp(3,1) + COMP_irho * p(3) * Grad0(1);
        Grad_v_gp(3,2)              = Grad_v_gp(3,2) + COMP_irho * p(3) * Grad0(2);        
        Grad_v_gp(3,3)              = Grad_v_gp(3,3) + COMP_irho * p(3) * Grad0(3);          
        
        Grad_Theta_gp(1)            = Grad_Theta_gp(1) + Theta * Grad0(1);
        Grad_Theta_gp(2)            = Grad_Theta_gp(2) + Theta * Grad0(2);
        Grad_Theta_gp(3)            = Grad_Theta_gp(3) + Theta * Grad0(3);        
    end

    %----------------------------------------------------------------------
    % Right-hand-side of area map evolution
    % "Tensor cross product" computation between F and gradient of velocity
    %----------------------------------------------------------------------
    F_Grad_v_gp                     = tensor_cross_product(Grad_v_gp, F_gp);
    
    %----------------------------------------------------------------------
    % Right-hand-side of volume map evolution
    %----------------------------------------------------------------------
    H_Grad_v_gp                     = H_gp(1,1) * Grad_v_gp(1,1) + H_gp(1,2) * Grad_v_gp(1,2) + H_gp(1,3) * Grad_v_gp(1,3) + ...
                                      H_gp(2,1) * Grad_v_gp(2,1) + H_gp(2,2) * Grad_v_gp(2,2) + H_gp(2,3) * Grad_v_gp(2,3) + ...
                                      H_gp(3,1) * Grad_v_gp(3,1) + H_gp(3,2) * Grad_v_gp(3,2) + H_gp(3,3) * Grad_v_gp(3,3);
    
    %----------------------------------------------------------------------
    % Heat flux
    %----------------------------------------------------------------------
%     Q_gp                            = - DAT_Conduct * H_gp' * H_gp/J_gp * Grad_Theta_gp;
    HTH                            = [0 0 0;0 0 0; 0 0 0];
    HTH(1,1)                       = H_gp(1,1) * H_gp(1,1) + H_gp(2,1) * H_gp(2,1) + H_gp(3,1) * H_gp(3,1);
    HTH(1,2)                       = H_gp(1,1) * H_gp(1,2) + H_gp(2,1) * H_gp(2,2) + H_gp(3,1) * H_gp(3,2);
    HTH(1,3)                       = H_gp(1,1) * H_gp(1,3) + H_gp(2,1) * H_gp(2,3) + H_gp(3,1) * H_gp(3,3);
    HTH(2,2)                       = H_gp(1,2) * H_gp(1,2) + H_gp(2,2) * H_gp(2,2) + H_gp(3,2) * H_gp(3,2);
    HTH(2,3)                       = H_gp(1,2) * H_gp(1,3) + H_gp(2,2) * H_gp(2,3) + H_gp(3,2) * H_gp(3,3);
    HTH(3,3)                       = H_gp(1,3) * H_gp(1,3) + H_gp(2,3) * H_gp(2,3) + H_gp(3,3) * H_gp(3,3);
    HTH(2,1)                       = HTH(1,2);
    HTH(3,1)                       = HTH(1,3);
    HTH(3,2)                       = HTH(2,3);
    Q_gp                           = [0;0;0];
    Q_gp(1)                        = - DAT_Conduct/J_gp * (HTH(1,1) * Grad_Theta_gp(1) + HTH(1,2) * Grad_Theta_gp(2) + HTH(1,3) * Grad_Theta_gp(3));
    Q_gp(2)                        = - DAT_Conduct/J_gp * (HTH(2,1) * Grad_Theta_gp(1) + HTH(2,2) * Grad_Theta_gp(2) + HTH(2,3) * Grad_Theta_gp(3));
    Q_gp(3)                        = - DAT_Conduct/J_gp * (HTH(3,1) * Grad_Theta_gp(1) + HTH(3,2) * Grad_Theta_gp(2) + HTH(3,3) * Grad_Theta_gp(3));    

    %----------------------------------------------------------------------
    % Loop over the number of nodes within an element
    %----------------------------------------------------------------------
    for in = 1:GEO_NodeE
        
        %------------------------------------------------------------------
        % Node id
        %------------------------------------------------------------------
        Node_ID                     = GEO_T(in,ielem);
        
        %------------------------------------------------------------------
        % Material gradient
        %------------------------------------------------------------------
%         Grad0                       = DAT_ISO_Grad0(in,:,ielem);
        Grad0                       = [0 0 0];
        Grad0(1)                    = DAT_ISO_Grad0(in,1,ielem);
        Grad0(2)                    = DAT_ISO_Grad0(in,2,ielem);
        Grad0(3)                    = DAT_ISO_Grad0(in,3,ielem);
        
        %------------------------------------------------------------------
        % Nodal contribution for fibre map, area map, volume map and
        % entropy equations
        %------------------------------------------------------------------
%         RF(:,:,Node_ID)    = RF(:,:,Node_ID)    + Grad_v_gp    / GEO_NodeE * J0gp;
%         RH(:,:,Node_ID)    = RH(:,:,Node_ID)    + F_Grad_v_gp  / GEO_NodeE * J0gp;
        RJ(Node_ID)        = RJ(Node_ID)        + H_Grad_v_gp  / GEO_NodeE * J0gp;
%         REta(Node_ID)      = REta(Node_ID)      + Grad0 * Q_gp / Theta_gp * J0gp - Grad_Theta_gp' * Q_gp / Theta_gp / Theta_gp * J0gp;
        REta(Node_ID)      = REta(Node_ID)      + (Grad0(1) * Q_gp(1) + Grad0(2) * Q_gp(2) + Grad0(3) * Q_gp(3)) / Theta_gp * J0gp - (Grad_Theta_gp(1) * Q_gp(1) + Grad_Theta_gp(2) * Q_gp(2) + Grad_Theta_gp(3) * Q_gp(3)) / Theta_gp / Theta_gp * J0gp;
        
        %------------------------------------------------------------------
        % Vectorisation
        %------------------------------------------------------------------
        RF(1,1,Node_ID)    = RF(1,1,Node_ID)    + Grad_v_gp(1,1)    / GEO_NodeE * J0gp;
        RF(1,2,Node_ID)    = RF(1,2,Node_ID)    + Grad_v_gp(1,2)    / GEO_NodeE * J0gp;
        RF(1,3,Node_ID)    = RF(1,3,Node_ID)    + Grad_v_gp(1,3)    / GEO_NodeE * J0gp;
        RF(2,1,Node_ID)    = RF(2,1,Node_ID)    + Grad_v_gp(2,1)    / GEO_NodeE * J0gp;
        RF(2,2,Node_ID)    = RF(2,2,Node_ID)    + Grad_v_gp(2,2)    / GEO_NodeE * J0gp;
        RF(2,3,Node_ID)    = RF(2,3,Node_ID)    + Grad_v_gp(2,3)    / GEO_NodeE * J0gp;
        RF(3,1,Node_ID)    = RF(3,1,Node_ID)    + Grad_v_gp(3,1)    / GEO_NodeE * J0gp;
        RF(3,2,Node_ID)    = RF(3,2,Node_ID)    + Grad_v_gp(3,2)    / GEO_NodeE * J0gp;
        RF(3,3,Node_ID)    = RF(3,3,Node_ID)    + Grad_v_gp(3,3)    / GEO_NodeE * J0gp;
     
        RH(1,1,Node_ID)    = RH(1,1,Node_ID)    + F_Grad_v_gp(1,1)  / GEO_NodeE * J0gp;
        RH(1,2,Node_ID)    = RH(1,2,Node_ID)    + F_Grad_v_gp(1,2)  / GEO_NodeE * J0gp;
        RH(1,3,Node_ID)    = RH(1,3,Node_ID)    + F_Grad_v_gp(1,3)  / GEO_NodeE * J0gp;
        RH(2,1,Node_ID)    = RH(2,1,Node_ID)    + F_Grad_v_gp(2,1)  / GEO_NodeE * J0gp;
        RH(2,2,Node_ID)    = RH(2,2,Node_ID)    + F_Grad_v_gp(2,2)  / GEO_NodeE * J0gp;
        RH(2,3,Node_ID)    = RH(2,3,Node_ID)    + F_Grad_v_gp(2,3)  / GEO_NodeE * J0gp;
        RH(3,1,Node_ID)    = RH(3,1,Node_ID)    + F_Grad_v_gp(3,1)  / GEO_NodeE * J0gp;
        RH(3,2,Node_ID)    = RH(3,2,Node_ID)    + F_Grad_v_gp(3,2)  / GEO_NodeE * J0gp;
        RH(3,3,Node_ID)    = RH(3,3,Node_ID)    + F_Grad_v_gp(3,3)  / GEO_NodeE * J0gp;
        
    end
    
end

end