function [RJ] = RHS_J_stabilisation(FUNC,UNK,GEO,DAT,COMP,RJ,Rp)

%--------------------------------------------------------------------------
% Extraction
%--------------------------------------------------------------------------
GEO_Nelem                        = GEO.Nelem;
GEO_NodeE                        = GEO.NodeE;
GEO_T                            = GEO.T;
DAT_ISO_J0gp                     = DAT.ISO_J0gp;
DAT_ISO_Grad0                    = DAT.ISO_Grad0;
DAT_Temp_R                       = DAT.Temp_R;
DAT_SUPG_taup                    = DAT.SUPG_taup;
UNK_F                            = UNK.F;
UNK_H                            = UNK.H;
UNK_J                            = UNK.J;
UNK_Eta                          = UNK.Eta;
GEO_h                            = GEO.h;
COMP_up_temp                     = COMP.up_temp;
COMP_irho                        = COMP.irho;
FUNC_Piola_kirchhoff             = FUNC.Piola_kirchhoff;

%--------------------------------------------------------------------------
% Loop over the element
%--------------------------------------------------------------------------
for ielem = 1:GEO_Nelem
    
    %----------------------------------------------------------------------
    % Initation
    %----------------------------------------------------------------------
    H_gp                                      = [0 0 0; 0 0 0; 0 0 0];  
    F_gp                                      = [0 0 0; 0 0 0; 0 0 0];      
    DIV_P_gp                                  = [0;0;0];
    pdot_gp                                   = [0;0;0];
    
    %----------------------------------------------------------------------
    % Elemental current volume
    %----------------------------------------------------------------------
    J0gp                                      = DAT_ISO_J0gp(ielem);
    
    %----------------------------------------------------------------------
    % Interpolated variables @ Gauss quadrature points
    %----------------------------------------------------------------------
    for in = 1:GEO_NodeE
        
        %------------------------------------------------------------------
        % Node id
        %------------------------------------------------------------------
        Node_ID                     = GEO_T(in,ielem);
        
        %------------------------------------------------------------------
        % Material gradient
        %------------------------------------------------------------------
%         Grad0                       = DAT_ISO_Grad0(in,:,ielem);
        Grad0                       = [0 0 0];
        Grad0(1)                    = DAT_ISO_Grad0(in,1,ielem);
        Grad0(2)                    = DAT_ISO_Grad0(in,2,ielem);
        Grad0(3)                    = DAT_ISO_Grad0(in,3,ielem);
        
        %------------------------------------------------------------------
        % Extract area map and the time rate of linear momentum
        %------------------------------------------------------------------
%         F                           = UNK_F(:,:,Node_ID);
%         H                           = UNK_H(:,:,Node_ID);
        J                           = UNK_J(Node_ID);
        Eta                         = UNK_Eta(Node_ID);
        ThetaR                      = DAT_Temp_R(Node_ID);
        pdot                        = Rp(:,Node_ID);  
        
        %------------------------------------------------------------------
        % Vectorisation
        %------------------------------------------------------------------
        F                           = [0 0 0; 0 0 0; 0 0 0];
        F(1,1)                      = UNK_F(1,1,Node_ID);
        F(1,2)                      = UNK_F(1,2,Node_ID);
        F(1,3)                      = UNK_F(1,3,Node_ID);        
        F(2,1)                      = UNK_F(2,1,Node_ID);
        F(2,2)                      = UNK_F(2,2,Node_ID);
        F(2,3)                      = UNK_F(2,3,Node_ID);   
        F(3,1)                      = UNK_F(3,1,Node_ID);
        F(3,2)                      = UNK_F(3,2,Node_ID);
        F(3,3)                      = UNK_F(3,3,Node_ID);   
        
        H                           = [0 0 0; 0 0 0; 0 0 0];
        H(1,1)                      = UNK_H(1,1,Node_ID);
        H(1,2)                      = UNK_H(1,2,Node_ID);
        H(1,3)                      = UNK_H(1,3,Node_ID);        
        H(2,1)                      = UNK_H(2,1,Node_ID);
        H(2,2)                      = UNK_H(2,2,Node_ID);
        H(2,3)                      = UNK_H(2,3,Node_ID);   
        H(3,1)                      = UNK_H(3,1,Node_ID);
        H(3,2)                      = UNK_H(3,2,Node_ID);
        H(3,3)                      = UNK_H(3,3,Node_ID);   
        
        %------------------------------------------------------------------
        % First Piola Kirchhoff stress tensor
        %------------------------------------------------------------------
        P                           = FUNC_Piola_kirchhoff(COMP,DAT,F,H,J,F,H,J,Eta,ThetaR);    

        %------------------------------------------------------------------
        % Variables at Gauss quadrature point
        %------------------------------------------------------------------
        H_gp                        = H_gp         + H    / GEO_NodeE;
        F_gp                        = F_gp         + F    / GEO_NodeE;
        pdot_gp                     = pdot_gp      + pdot / GEO_NodeE;
%         DIV_P_gp                    = DIV_P_gp     + P * Grad0';     
        
        %------------------------------------------------------------------
        % Vectorisation
        %------------------------------------------------------------------
        DIV_P_gp(1)                 = DIV_P_gp(1)  + P(1,1) * Grad0(1) + P(1,2) * Grad0(2) + P(1,3) * Grad0(3); 
        DIV_P_gp(2)                 = DIV_P_gp(2)  + P(2,1) * Grad0(1) + P(2,2) * Grad0(2) + P(2,3) * Grad0(3); 
        DIV_P_gp(3)                 = DIV_P_gp(3)  + P(3,1) * Grad0(1) + P(3,2) * Grad0(2) + P(3,3) * Grad0(3);         
    end

    %----------------------------------------------------------------------
    % Local time increment
    %----------------------------------------------------------------------
%     [dt]                            = Local_dt(F_gp,GEO_h(ielem),COMP_up_temp);
    dt                              = GEO_h(ielem) / COMP_up_temp;
    
    %----------------------------------------------------------------------
    % PG stabilisation terms associated with the volume map evolution
    %----------------------------------------------------------------------
    RJ_Aux                          = DIV_P_gp - pdot_gp;
    
    %----------------------------------------------------------------------
    % Loop over the number of nodes within an element
    %----------------------------------------------------------------------
    for in = 1:GEO_NodeE
        
        %------------------------------------------------------------------
        % Node id
        %------------------------------------------------------------------
        Node_ID                     = GEO_T(in,ielem);
        
        %------------------------------------------------------------------
        % Material gradient
        %------------------------------------------------------------------
%         Grad0                       = DAT_ISO_Grad0(in,:,ielem);
        Grad0                       = [0 0 0];
        Grad0(1)                    = DAT_ISO_Grad0(in,1,ielem);
        Grad0(2)                    = DAT_ISO_Grad0(in,2,ielem);
        Grad0(3)                    = DAT_ISO_Grad0(in,3,ielem);
        
        %------------------------------------------------------------------
        % Nodal contribution for the volume map equation
        %------------------------------------------------------------------
%         RJ(Node_ID)                 = RJ(Node_ID) - COMP_irho * DAT_SUPG_taup * dt *  Grad0 * (H_gp' * RJ_Aux) * J0gp;
        HTRJ                        = [0;0;0];
        HTRJ(1)                     = H_gp(1,1)*RJ_Aux(1) + H_gp(2,1)*RJ_Aux(2) + H_gp(3,1)*RJ_Aux(3);
        HTRJ(2)                     = H_gp(1,2)*RJ_Aux(1) + H_gp(2,2)*RJ_Aux(2) + H_gp(3,2)*RJ_Aux(3);
        HTRJ(3)                     = H_gp(1,3)*RJ_Aux(1) + H_gp(2,3)*RJ_Aux(2) + H_gp(3,3)*RJ_Aux(3);
        RJ(Node_ID)                 = RJ(Node_ID) - COMP_irho * DAT_SUPG_taup * dt *  (Grad0(1) * HTRJ(1) + Grad0(2) * HTRJ(2) + Grad0(3) * HTRJ(3)) * J0gp;
        
    end
    
end

end