%--------------------------------------------------------------------------
% Evaluation of the right hand side of all the evolutions
%--------------------------------------------------------------------------
function  [UNK,COMP] = RHS_all(UNK,COMP,GEO,DAT,FUNC)

%--------------------------------------------------------------------------
% Right hande side of {F, H, J} and entropy equation
% No stabilisation terms are needed for these two evolutions
%--------------------------------------------------------------------------
[RF,REta]                        = RHS_F_and_Eta(UNK,GEO,DAT,COMP);

%--------------------------------------------------------------------------
% Evaluation of the time rate of F and Eta
% These will be used for stabilisation terms in linear momentum evolution
%--------------------------------------------------------------------------
Fdot                            = Solve_System(RF,GEO,COMP);
Etadot                          = REta ./ COMP.mass;

%--------------------------------------------------------------------------
% Right-hand sides of linear momentum conservation law 
%--------------------------------------------------------------------------
[Rp,REta]                       = RHS_p(FUNC,UNK,GEO,DAT,COMP,Fdot,Etadot);

%--------------------------------------------------------------------------
% Global angular momentum projection through least square minisation procedure
%--------------------------------------------------------------------------
[Rp]                            = Angular_Momentum_Projection(Rp,COMP.mass,UNK.X_AM,GEO.Nnode);

%--------------------------------------------------------------------------
% Boundary contribution for linear momentum and fibre map equations
%--------------------------------------------------------------------------
[Rp]                            = Boundary_Contribution(FUNC,UNK,GEO,DAT,COMP,Rp);

%--------------------------------------------------------------------------
% Evaluation of the time rate of p
%--------------------------------------------------------------------------
pdot                            = zeros(3,GEO.Nnode);
pdot(1,:)                       = Rp(1,:) ./ COMP.mass';
pdot(2,:)                       = Rp(2,:) ./ COMP.mass';
pdot(3,:)                       = Rp(3,:) ./ COMP.mass';

%--------------------------------------------------------------------------
% Update mesh coordinates
%--------------------------------------------------------------------------
UNK.X                           = UNK.X + COMP.dt * COMP.irho * UNK.p;

%--------------------------------------------------------------------------
% Evolve unknown variables to next time step
%--------------------------------------------------------------------------
UNK.p                           = UNK.p     + pdot * COMP.dt;
UNK.F                           = UNK.F     + Fdot * COMP.dt;
UNK.Eta                         = UNK.Eta   + Etadot * COMP.dt;
 
%--------------------------------------------------------------------------
% Impose strong boundary conditions on linear momentum
%--------------------------------------------------------------------------
[UNK]                           = Apply_strong_boundary_conditions(UNK,GEO);

end
