function [Rp,REta] = RHS_p(FUNC,UNK,GEO,DAT,COMP,RF,REta)

%--------------------------------------------------------------------------
% Extraction
%--------------------------------------------------------------------------
GEO_Nelem         = GEO.Nelem;
GEO_Nnode         = GEO.Nnode;
GEO_NodeE         = GEO.NodeE;
GEO_T             = GEO.T;
GEO_h             = GEO.h;
DAT_ISO_Grad0     = DAT.ISO_Grad0;
DAT_ISO_J0gp      = DAT.ISO_J0gp;
DAT_SUPG_tauF     = DAT.SUPG_tauF;
DAT_SUPG_xiF      = DAT.SUPG_xiF;
DAT_Temp_R        = DAT.Temp_R;
DAT_MG_gamma      = DAT.MG_gamma;
DAT_MG_q          = DAT.MG_q;
DAT_Den           = DAT.Den;
DAT_CF            = DAT.CF;
DAT_SHOCK_taup    = DAT.SHOCK_taup;
DAT_SHOCK_TOL     = DAT.SHOCK_TOL; 
COMP_mu           = COMP.mu;
COMP_kappa        = COMP.kappa;
COMP_up_temp      = COMP.up_temp;
COMP_irho         = COMP.irho;
UNK_p             = UNK.p;
UNK_F             = UNK.F;
UNK_Eta           = UNK.Eta;
UNK_X             = UNK.X;
FUNC_Piola_kirchhoff = FUNC.Piola_kirchhoff;

%--------------------------------------------------------------------------
% Initiation
%--------------------------------------------------------------------------
Rp                = zeros(3,GEO_Nnode);

%--------------------------------------------------------------------------
% Loop over the element
%--------------------------------------------------------------------------
for ielem = 1:GEO_Nelem
    
    %----------------------------------------------------------------------
    % Initation
    %----------------------------------------------------------------------
    F_gp                            = [0 0 0; 0 0 0; 0 0 0];
    Eta_gp                          = 0;
    Fdot_gp                         = [0 0 0; 0 0 0; 0 0 0];
    Fx_gp                           = [0 0 0; 0 0 0; 0 0 0];
    Grad_v_gp                       = [0 0 0; 0 0 0; 0 0 0];
    ThetaR_gp                       = 0;
    Grad_normv_gp                   = [0;0;0];
    Theta_gp                        = 0;
    Eta_Prime_gp                    = 0;
    Grad_J_gp                       = [0;0;0];
    
    %----------------------------------------------------------------------
    % Elemental current volume
    %----------------------------------------------------------------------
    J0gp                            = DAT_ISO_J0gp(ielem);
    
    %----------------------------------------------------------------------
    % Interpolated variables @ Gauss quadrature points
    %----------------------------------------------------------------------
    for in = 1:GEO_NodeE
        
        %------------------------------------------------------------------
        % Node ID
        %------------------------------------------------------------------
        Node_ID                     = GEO_T(in,ielem);
        
        %------------------------------------------------------------------
        % Material gradient
        %------------------------------------------------------------------
        Grad0                       = DAT_ISO_Grad0(in,:,ielem);
        
        %------------------------------------------------------------------
        % Extract all variables
        %------------------------------------------------------------------
        p                           = UNK_p(:,Node_ID);
        F                           = UNK_F(:,:,Node_ID);
        Fdot                        = RF(:,:,Node_ID);
        Eta                         = UNK_Eta(Node_ID);
        X                           = UNK_X(:,Node_ID);
        ThetaR                      = DAT_Temp_R(Node_ID);
        v                           = COMP_irho * p;
        norm_v                      = sqrt(v' * v);
        J                           = det(F);
        
        %------------------------------------------------------------------
        % Thermal variables
        %------------------------------------------------------------------
        Theta0                      = DAT_Temp_R(Node_ID) * exp(-DAT_MG_gamma*(J^(DAT_MG_q)-1)/DAT_MG_q);
        Theta                       = Theta0 * exp(Eta/DAT_Den/DAT_CF);
        Eta_Prime                   = DAT_Den * DAT_CF * DAT_MG_gamma * J^(DAT_MG_q-1);    
                
        %------------------------------------------------------------------
        % variables at Gauss quadrature point
        %------------------------------------------------------------------
        F_gp                        = F_gp         + F / GEO_NodeE;
        Eta_gp                      = Eta_gp       + Eta / GEO_NodeE;
        Fdot_gp                     = Fdot_gp      + Fdot / GEO_NodeE;
        Fx_gp                       = Fx_gp        + X    * Grad0;
        Grad_v_gp                   = Grad_v_gp    + COMP_irho * p * Grad0;
        ThetaR_gp                   = ThetaR_gp    + ThetaR / GEO_NodeE;
        
%         Grad_normv_gp(1)            = Grad_normv_gp(1) + norm_v * Grad0(1);
%         Grad_normv_gp(2)            = Grad_normv_gp(2) + norm_v * Grad0(2);
%         Grad_normv_gp(3)            = Grad_normv_gp(3) + norm_v * Grad0(3);
%         
% %         Theta_gp                    = Theta_gp       + Theta / GEO_NodeE;
% %         Eta_Prime_gp                = Eta_Prime_gp   + Eta_Prime / GEO_NodeE;
%        
%         Grad_J_gp(1)                = Grad_J_gp(1) + J * Grad0(1);
%         Grad_J_gp(2)                = Grad_J_gp(2) + J * Grad0(2);
%         Grad_J_gp(3)                = Grad_J_gp(3) + J * Grad0(3);
        
    end
    
    %----------------------------------------------------------------------
    % Local time increment
    %----------------------------------------------------------------------
    dt                              = GEO_h(ielem) / COMP_up_temp;
   
    %----------------------------------------------------------------------
    % Enhanced geometric strain variables {F, H, J}
    %----------------------------------------------------------------------
    H_gp                            = 0.5 * tensor_cross_product(F_gp,F_gp);
    J_gp                            = det(F_gp);
    F_st                            = F_gp + DAT_SUPG_tauF * dt * (Grad_v_gp - Fdot_gp) + DAT_SUPG_xiF * (Fx_gp - F_gp);
    H_st                            = 0.5 * tensor_cross_product(F_st,F_st);
    J_st                            = det(F_st);

    %----------------------------------------------------------------------
    % First Piola Kirchhoff stress tensor
    %----------------------------------------------------------------------
    [P_gp]                          = FUNC_Piola_kirchhoff(COMP,DAT,F_gp,H_gp,J_gp,F_st,H_st,J_st,Eta_gp,ThetaR_gp);
    
%     %----------------------------------------------------------------------
%     % Shock operator
%     %----------------------------------------------------------------------
%     inv_F                           = F_gp^(-1);
%     [P_gp,REta_Shock]               = Shock_opeator(P_gp,inv_F'*Grad_normv_gp,Grad_v_gp,DAT_SHOCK_taup,GEO_h(ielem),COMP_up_temp,DAT_SHOCK_TOL,COMP_mu,Grad_J_gp,Eta_Prime_gp);
%     dP_gp                           = P_gp - Pe_gp;
    
    %----------------------------------------------------------------------
    % Loop over the number of nodes within an element
    %----------------------------------------------------------------------
    for in = 1:GEO_NodeE
        
        %------------------------------------------------------------------
        % Node ID
        %------------------------------------------------------------------
        Node_ID                     = GEO_T(in,ielem);
        
        %------------------------------------------------------------------
        % Material gradient
        %------------------------------------------------------------------
        Grad0                       = DAT_ISO_Grad0(in,:,ielem);
        
        %------------------------------------------------------------------
        % Nodal contribution for linear momentum equation
        %------------------------------------------------------------------       
        Rp(:,Node_ID)              = Rp(:,Node_ID) - (P_gp * Grad0') * J0gp;
              
    end
    
end

end