%--------------------------------------------------------------------------
% One step two stages TVD-RK time integrator
% Solver in a sequence manner
%--------------------------------------------------------------------------
function  [UNK,COMP]  = TVD_RK(UNK,COMP,GEO,FUNC,DAT)

%--------------------------------------------------------------------------
% Save old variables
% Linear momentum, Fibre map and entropy 
% Position and time
%--------------------------------------------------------------------------
UNK.pold                        = UNK.p;
UNK.Fold                        = UNK.F;
UNK.Etaold                      = UNK.Eta;
UNK.Xold                        = UNK.X;

%--------------------------------------------------------------------------
% Evaluation of the right hand side of all the evolutions
% First stage of TVD Runge Kutta time integrator
%--------------------------------------------------------------------------
UNK.X_AM                        = UNK.X;    % Coordinates to be used in the local angular momentum projection
[UNK,COMP]                      = RHS_all(UNK,COMP,GEO,DAT,FUNC);

%--------------------------------------------------------------------------
% Evaluation of the right hand side of all the evolutions
% Second stage of TVD Runge Kutta time integrator
%--------------------------------------------------------------------------
UNK.X_AM                        = UNK.Xold + 0.5 * COMP.dt * COMP.irho * (UNK.pold + UNK.p); % Coordinates to be used in the local angular momentum projection
[UNK,COMP]                      = RHS_all(UNK,COMP,GEO,DAT,FUNC);

%--------------------------------------------------------------------------
% Unknown variables @ next time step (n+1)
% Linear momentum and all the geometric strain variables {F,H,J}
% Position and actual time
%--------------------------------------------------------------------------
UNK.p                           = 0.5 * (UNK.pold + UNK.p);
UNK.F                           = 0.5 * (UNK.Fold + UNK.F);
UNK.Eta                         = 0.5 * (UNK.Etaold + UNK.Eta);
UNK.X                           = 0.5 * (UNK.Xold + UNK.X);
COMP.time                       = COMP.time + COMP.dt;

%--------------------------------------------------------------------------
% Impose strong boundary conditions on linear momentum
%--------------------------------------------------------------------------
[UNK]                           = Apply_strong_boundary_conditions(UNK,GEO);

end