%--------------------------------------------------------------------------
% Nearly incompressible neo-Hookean model
%--------------------------------------------------------------------------
function [P] = Nearly_incompressible_neo_Hookean(COMP,DAT,F,H,J,Fst,Hst,Jst,Eta,ThetaR)

%--------------------------------------------------------------------------
% Preliminaries
%--------------------------------------------------------------------------
COMP_mu            = COMP.mu;
COMP_kappa         = COMP.kappa; 

%--------------------------------------------------------------------------
% Jacobian
%--------------------------------------------------------------------------
JFst                 = det(Fst);
                  
%--------------------------------------------------------------------------
% Area map tensor
%--------------------------------------------------------------------------
HF                   = 0.5 * tensor_cross_product(F,F);

%--------------------------------------------------------------------------
% Trace of Right Cauchy Green tensor
%--------------------------------------------------------------------------
TrCst                = Fst(1,1)*Fst(1,1) + Fst(2,1)*Fst(2,1) + Fst(3,1)*Fst(3,1) + ...
                       Fst(1,2)*Fst(1,2) + Fst(2,2)*Fst(2,2) + Fst(3,2)*Fst(3,2) + ...
                       Fst(1,3)*Fst(1,3) + Fst(2,3)*Fst(2,3) + Fst(3,3)*Fst(3,3);
                 
%--------------------------------------------------------------------------
% Conjugate stresses
%--------------------------------------------------------------------------
SigmaF               = COMP_mu   * JFst^(-2/3) * Fst;
SigmaJ_Hat           = -COMP_mu/3 * JFst^(-5/3) * TrCst;
Pressure             = COMP_kappa * (JFst - 1);
SigmaJ               = SigmaJ_Hat + Pressure;

%--------------------------------------------------------------------------
% First Piola
%--------------------------------------------------------------------------
P                    = SigmaF + SigmaJ * HF; 

end