%--------------------------------------------------------------------------
% Output file to Paraview
%--------------------------------------------------------------------------
function [OUT] = Output_Paraview(OUT,outputfile,COMP,GEO)

%--------------------------------------------------------------------------
% Open file
%--------------------------------------------------------------------------
outputfid=fopen(outputfile,'w');

%--------------------------------------------------------------------------
% Print statement
%--------------------------------------------------------------------------
fprintf(outputfid,'# vtk DataFile Version 3.1\n');
fprintf(outputfid,'SUPG-SPH Mixed Formulation\n');
fprintf(outputfid,'ASCII\n');
fprintf(outputfid,'DATASET UNSTRUCTURED_GRID\n');

%--------------------------------------------------------------------------
% Time
%--------------------------------------------------------------------------
fprintf(outputfid,'\nFIELD FieldData %d \n', 1);
fprintf(outputfid,'TIME %d %d double\n', 1, 1);
fprintf(outputfid,'%d\n', COMP.time);

%--------------------------------------------------------------------------
% Spatial coordinates
%--------------------------------------------------------------------------
fprintf(outputfid,'\nPOINTS %d FLOAT \n', size(OUT.UNK_X,2));
for inode=1:size(OUT.UNK_X,2)
    fprintf(outputfid,'%5.7f %5.7f %5.7f\n',...
        OUT.UNK_X(:,inode));
end

%--------------------------------------------------------------------------
% Connectivities
%--------------------------------------------------------------------------
fprintf(outputfid,'\nCELLS %d %d\n', GEO.Nelem, GEO.Nelem * (GEO.NodeE + 1));
for inode=1:GEO.Nelem
    fprintf(outputfid,'%d %d %d %d %d \n',...
        GEO.NodeE, GEO.T(:,inode)-1);
end

%--------------------------------------------------------------------------
% Cell types
%--------------------------------------------------------------------------
Cell_type = zeros(1,GEO.Nelem);
Cell_type(:) = 10;
fprintf(outputfid,'\nCELL_TYPES %d \n', GEO.Nelem);
fprintf(outputfid,'%d ', Cell_type);

%--------------------------------------------------------------------------
% Pressure variables
%--------------------------------------------------------------------------
fprintf(outputfid,'\n \nPOINT_DATA %d \n', size(OUT.UNK_X,2));
fprintf(outputfid,'SCALARS Pressure FLOAT\n');
fprintf(outputfid,'LOOKUP_TABLE default \n');
for inode=1:size(OUT.UNK_X,2)
    fprintf(outputfid,'%5.7f \n',OUT.UNK_Pressure(inode));
%     fprintf(outputfid,'%5.7f \n',OUT.UNK_Theta(inode));
end

% % %--------------------------------------------------------------------------
% % % Temperature
% % %--------------------------------------------------------------------------
% % fprintf(outputfid,'\n \nPOINT_DATA %d \n', size(OUT.UNK_X,2));
% % fprintf(outputfid,'SCALARS Theta FLOAT\n');
% fprintf(outputfid,'LOOKUP_TABLE %d \n', size(OUT.UNK_X,2));
% for inode=1:size(OUT.UNK_X,2)
%     fprintf(outputfid,'%5.7f \n',OUT.UNK_Theta(inode));
% end

%--------------------------------------------------------------------------
% Close outputfile
%--------------------------------------------------------------------------
fclose(outputfid);

end