%--------------------------------------------------------------------------
% Plot time history of Angular momentum
%--------------------------------------------------------------------------
function Evolution_Angular_momentum

%--------------------------------------------------------------------------
% Close all
%--------------------------------------------------------------------------
close all

%--------------------------------------------------------------------------
% Load result file
%--------------------------------------------------------------------------
load('Mesh.mat');
load('Result.mat');

%--------------------------------------------------------------------------
% Initiation
%--------------------------------------------------------------------------
k=0;
%--------------------------------------------------------------------------
% Extract momentum results
%--------------------------------------------------------------------------
NStep = 1400;
Interval = 10;
Angular_momentum_History = zeros(3,NStep/Interval);

for i=1:Interval: NStep
    
    %----------------------------------------------------------------------
    % Snapshots
    %----------------------------------------------------------------------
    k = k + 1;
    
    %----------------------------------------------------------------------
    % File name
    %----------------------------------------------------------------------
    s                        = sprintf('snapshot%010d.mat',i-1);
    
    %----------------------------------------------------------------------
    % Load the corresponding file
    %----------------------------------------------------------------------
    load(s);
    
    %----------------------------------------------------------------------
    % Store time history of relevant variables
    %----------------------------------------------------------------------
    for j=1:GEO.Nnode
    Angular_momentum_History(:,k) = Angular_momentum_History(:,k)+...
                                    cross((OUT.UNK_X(:,j)),OUT.UNK_p(:,j))*COMP.mass(j);
    end
    Time_History(k,1) = OUT.Time;
end

hold off

%--------------------------------------------------------------------------
% Plot linear momentum
%--------------------------------------------------------------------------
figure(1)
plot(Time_History,Angular_momentum_History(1,:),'ro-');
hold on
plot(Time_History,Angular_momentum_History(2,:),'k*-');
hold on
plot(Time_History,Angular_momentum_History(3,:),'bx-');
set(gca,'DefaultTextInterpreter','latex','fontsize',20)

ylabel('Angular Momentum (N.m.s$^{-1}$)','Interpreter','latex','FontSize',20)
xlabel('Time(s)','Interpreter','latex','FontSize',20)
h = legend('Angular Momentum X-X','Angular Momentum Y-Y','Angular Momentum Z-Z');
set(h,'Interpreter','latex','FontSize',20);
grid off

end