%--------------------------------------------------------------------------
% Plot time history of Linear momentum
%--------------------------------------------------------------------------
function Evolution_Dissipation

%--------------------------------------------------------------------------
% Close all
%--------------------------------------------------------------------------
close all

%--------------------------------------------------------------------------
% Load result file
%--------------------------------------------------------------------------
load('Mesh.mat');

%--------------------------------------------------------------------------
% Initiation
%--------------------------------------------------------------------------
k=0;
%--------------------------------------------------------------------------
% Extract momentum results
%--------------------------------------------------------------------------
NStep = 274;
iStep = 1;
Dissipation_History = zeros(3,NStep/iStep);
Time_History        = zeros(1,NStep/iStep);
Internal_Force_History = Dissipation_History;

for i=1:NStep/iStep
    
    %----------------------------------------------------------------------
    % Snapshots
    %----------------------------------------------------------------------
    k = k + 1;
    
    %----------------------------------------------------------------------
    % File name
    %----------------------------------------------------------------------
    s                        = sprintf('snapshot%010d.mat',CON.OutInc*(i-1));
    
    %----------------------------------------------------------------------
    % Load the corresponding file
    %----------------------------------------------------------------------
    load(s);
    
    %----------------------------------------------------------------------
    % Store time history of relevant variables
    %----------------------------------------------------------------------
    for j=1:GEO.NP
        Dissipation_History(:,k)     = Dissipation_History(:,k)+...
                                       OUT.Stab_p(:,j) * GEO.Vol(j);
        Internal_Force_History(:,k) = Internal_Force_History(:,k)+...
                                      OUT.Internal_Force_p(:,j) * GEO.Vol(j);
    end
    Time_History(k) = OUT.Time;
end


hold off

%--------------------------------------------------------------------------
% Plot linear momentum
%--------------------------------------------------------------------------
figure(1)
plot(Time_History,Dissipation_History(1,:),'ro-');
hold on
plot(Time_History,Dissipation_History(2,:),'k*-');
hold on
plot(Time_History,Dissipation_History(3,:),'bx-');
set(gca,'DefaultTextInterpreter','latex','fontsize',20)
ylabel('Dissipation (kg.m.s$^{-1}$)','Interpreter','latex','FontSize',20)
xlabel('Time(s)','Interpreter','latex','FontSize',20)
h = legend('Dissipation X-X','Dissipation Y-Y','Dissipation Z-Z');
set(h,'Interpreter','latex','FontSize',20);
grid off

figure(2)
plot(Time_History,Internal_Force_History(1,:),'ro-');
hold on
plot(Time_History,Internal_Force_History(2,:),'k*-');
hold on
plot(Time_History,Internal_Force_History(3,:),'bx-');
set(gca,'DefaultTextInterpreter','latex','fontsize',20)
% ylabel('Dissipation (kg.m.s$^{-1}$)','Interpreter','latex','FontSize',20)
ylabel('Internal Force','Interpreter','latex','FontSize',20)
xlabel('Time(s)','Interpreter','latex','FontSize',20)
h = legend('X-X','Y-Y','Z-Z');
set(h,'Interpreter','latex','FontSize',20);
grid off

end