function [CON,FUNC,DAT] = Read_Control_File

%--------------------------------------------------------------------------
% Open File
%--------------------------------------------------------------------------
meshfilename                  = 'Control.txt';
inputfid                      = fopen(meshfilename,'r');

%--------------------------------------------------------------------------
% Choose starting option
% [1] Start a new run
% [2] Restart
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
CON.Restart_Run               = deal(tmp);
 
%--------------------------------------------------------------------------
% Constitutive model
% [1] {p,F} Nearly incompressible Mooney Rivlin model
% [2] Thermo-elasticity
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
DAT.Model                     = deal(tmp);

%--------------------------------------------------------------------------
% Specific heat
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
DAT.CF                        = deal(tmp);

%--------------------------------------------------------------------------
% Mie Gruneisen parameter
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
DAT.MG_gamma                  = deal(tmp);
tmp                           = str2num(fgets(inputfid));
DAT.MG_q                      = deal(tmp);

%--------------------------------------------------------------------------
% Thermal conductivity
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
DAT.Conduct                   = deal(tmp);

%--------------------------------------------------------------------------
% Young's modulus
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
DAT.E                         = deal(tmp);

%--------------------------------------------------------------------------
% Poisson's ratio
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
DAT.Poisson                   = deal(tmp);

%--------------------------------------------------------------------------
% Density
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
DAT.Den                       = deal(tmp);

%--------------------------------------------------------------------------
% Number of time steps
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
CON.Nstep                     = deal(tmp);

%--------------------------------------------------------------------------
% Computation of time increment
% [1] Fixed CFL
% [2] Fixed Delta t
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
CON.Type_dt                   = deal(tmp);

%--------------------------------------------------------------------------
% Type of time increment
%--------------------------------------------------------------------------
switch CON.Type_dt
    case (1)
        FUNC.dt_method        = @fixed_cfl;
    case (2)
        FUNC.dt_method        = @fixed_dt;        
end

%--------------------------------------------------------------------------
% CFL coefficient
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
CON.CFL                       = deal(tmp);

%--------------------------------------------------------------------------
% Time increment
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
CON.dt                        = deal(tmp);

%--------------------------------------------------------------------------
% SUPG coefficient for F
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
DAT.SUPG_tauF                 = deal(tmp);
tmp                           = str2num(fgets(inputfid));
DAT.SUPG_xiF                  = deal(tmp);

%--------------------------------------------------------------------------
% Shock coefficient for p
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
DAT.SHOCK_taup                = deal(tmp);
tmp                           = str2num(fgets(inputfid));
DAT.SHOCK_TOL                 = deal(tmp);

%--------------------------------------------------------------------------
% Output results
%--------------------------------------------------------------------------
Title                         = fgets(inputfid);
tmp                           = str2num(fgets(inputfid));
CON.OutInc                    = deal(tmp);

%--------------------------------------------------------------------------
% Close file
%--------------------------------------------------------------------------
fclose(inputfid);


end