%--------------------------------------------------------------------------
% Characteristic length
%--------------------------------------------------------------------------
function [GEO_h]  = Characteristic_Length(GEO)

%--------------------------------------------------------------------------
% Extraction
%--------------------------------------------------------------------------
GEO_Nelem       = GEO.Nelem;
GEO_T           = GEO.T;
GEO_NodeE       = GEO.NodeE;
GEO_X           = GEO.X;
GEO_NodeF       = GEO.NodeF;

%--------------------------------------------------------------------------
% Initiation
%--------------------------------------------------------------------------
GEO_h           = zeros(1,GEO_Nelem);

for ielem = 1:GEO_Nelem
    
    %----------------------------------------------------------------------
    % Element connectivity
    %----------------------------------------------------------------------
    T  = GEO_T(:,ielem);
    
    %----------------------------------------------------------------------
    % Extract local nodes belong to an element
    %----------------------------------------------------------------------
    iA = T(1);
    iB = T(2);
    iC = T(3);
    iD = T(4);
 
    %----------------------------------------------------------------------
    % Find the characteristic distance in a tetrahedra
    %----------------------------------------------------------------------
    Edges_element = {[iA,iB,iC,iD];...
                     [iA,iC,iD,iB];...
                     [iA,iB,iD,iC];...
                     [iB,iC,iD,iA]};
                 
    %----------------------------------------------------------------------
    % Initiation
    %----------------------------------------------------------------------
    DistNodeCentr  = zeros(1,GEO_NodeE);
                 
    %----------------------------------------------------------------------
    % Charactestic length
    %----------------------------------------------------------------------
    for in = 1:GEO_NodeE
       
        %------------------------------------------------------------------
        % Node ID
        %------------------------------------------------------------------
        iA = Edges_element{in}(1);
        iB = Edges_element{in}(2);
        iC = Edges_element{in}(3);
        iD = Edges_element{in}(4);
        
        %------------------------------------------------------------------
        % Geometry
        %------------------------------------------------------------------
        XA = GEO_X(:,iA);
        XB = GEO_X(:,iB);
        XC = GEO_X(:,iC);
        XD = GEO_X(:,iD);
        
        %------------------------------------------------------------------
        % Face centroid
        %------------------------------------------------------------------
        CooCentrFace = (XA + XB + XC) / GEO_NodeF;
        
        %------------------------------------------------------------------
        % Distance from face centre to opposite node
        %------------------------------------------------------------------
        DistNodeCentr(1,in) = sqrt((CooCentrFace(1)-XD(1))^2+(CooCentrFace(2)-XD(2))^2+(CooCentrFace(3)-XD(3))^2);
                 
    end
    
    %----------------------------------------------------------------------
    % Elemental characteristic length
    %----------------------------------------------------------------------
    GEO_h(1,ielem) = min(DistNodeCentr);
    
end

end