%--------------------------------------------------------------------------
% Compute local dt
%--------------------------------------------------------------------------
function [dtime]= Local_dt(F,h,up_temp)

%--------------------------------------------------------------------------
% Right Cauchy Green tensor
%--------------------------------------------------------------------------
% C_gp                                      = F_gp' * F_gp;
C_gp                                      = zeros(3,3);
C_gp(1,1)                                 = F(1,1) * F(1,1) + F(2,1) * F(2,1) + F(3,1) * F(3,1); 
C_gp(1,2)                                 = F(1,1) * F(1,2) + F(2,1) * F(2,2) + F(3,1) * F(3,2); 
C_gp(1,3)                                 = F(1,1) * F(1,3) + F(2,1) * F(2,3) + F(3,1) * F(3,3); 
C_gp(2,2)                                 = F(1,2) * F(1,2) + F(2,2) * F(2,2) + F(3,2) * F(3,2); 
C_gp(2,3)                                 = F(1,2) * F(1,3) + F(2,2) * F(2,3) + F(3,2) * F(3,3); 
C_gp(3,3)                                 = F(1,3) * F(1,3) + F(2,3) * F(2,3) + F(3,3) * F(3,3); 
C_gp(2,1)                                 = C_gp(1,2);
C_gp(3,1)                                 = C_gp(1,3);
C_gp(3,2)                                 = C_gp(2,3);

%--------------------------------------------------------------------------
% Eigenvalues of Right Cauchy Green tensor
%--------------------------------------------------------------------------
% Stret                                     = eig(C_gp);
[Stret1,Stret2,Stret3]                    = Eigenvalue_Computation(C_gp(1,1),C_gp(1,2),C_gp(1,3),C_gp(2,2),C_gp(2,3),C_gp(3,3));
Stret1                                    = sqrt(Stret1);
Stret2                                    = sqrt(Stret2);
Stret3                                    = sqrt(Stret3);

%--------------------------------------------------------------------------
% Stretch 
%--------------------------------------------------------------------------
% PWave                                     = min(sqrt(Stret));
% PWave                                     = min([Stret1,Stret2,Stret3]);
PWave                                     = min(Stret1,Stret2);
PWave                                     = min(PWave,Stret3);

%--------------------------------------------------------------------------
% Volumetric wave speed
%--------------------------------------------------------------------------
PWave                                     = up_temp / PWave;

%--------------------------------------------------------------------------
% Local time increment
%--------------------------------------------------------------------------
dtime                                     = h / PWave;

end