%--------------------------------------------------------------------------
% Compute area and normal vector
%--------------------------------------------------------------------------
function [GEO_A_B,GEO_N_B] = area_normal_vector_HEX(GEO)

%--------------------------------------------------------------------------
% Extraction
%--------------------------------------------------------------------------
GEO_Nface      = GEO.Nface;
GEO_X          = GEO.X;
GEO_T_B        = GEO.T_B;
GEO_NodeF      = GEO.NodeF;

%--------------------------------------------------------------------------
% Initiation
%--------------------------------------------------------------------------
GEO_A_B        = zeros(1,GEO_Nface);
GEO_N_B        = zeros(3,GEO_Nface);

%--------------------------------------------------------------------------
% Information on linear triangular element 
%--------------------------------------------------------------------------
Gradxi0N(:,1)  = 1/4 * [-1 1 1 -1];
Gradxi0N(:,2)  = 1/4 * [-1 -1 1 1];
J0gp           = 4;

%--------------------------------------------------------------------------
% Loop over the boundary faces
%--------------------------------------------------------------------------
for iB = 1:GEO_Nface
    
    %----------------------------------------------------------------------
    % Extract local coordinates per element
    %----------------------------------------------------------------------
    Xelem                         = GEO_X(:,GEO_T_B(:,iB));
    
    %----------------------------------------------------------------------
    % Compute area normal vector
    %----------------------------------------------------------------------
    [Area,Normal]                 = Area_normal_vector(GEO_NodeF,Xelem,Gradxi0N);

    %----------------------------------------------------------------------
    % Store variables
    %----------------------------------------------------------------------
    GEO_A_B(1,iB)                 = Area * J0gp;   % Divided by 2 (isoparametric mapping with 1 Gauss quadrature point)
    GEO_N_B(:,iB)                 = Normal;
    
end

end


%--------------------------------------------------------------------------
% Compute area normal vector
%--------------------------------------------------------------------------
function [Area,Normal]  = Area_normal_vector(NodeF,Xelem,Gradxi)

%--------------------------------------------------------------------------
% Matrix
%--------------------------------------------------------------------------
Matrix           = [0 0 0; 0 0 0; 0 0 0];

for iB = 1:NodeF
    
   %-----------------------------------------------------------------------
   % GradxiX and GradetaX
   %-----------------------------------------------------------------------
   Matrix(:,1)   = Matrix(:,1) + Xelem(:,iB) * Gradxi(iB,1);
   Matrix(:,2)   = Matrix(:,2) + Xelem(:,iB) * Gradxi(iB,2);
    
end

%--------------------------------------------------------------------------
% Area vector
%--------------------------------------------------------------------------
Area_vector      = [0;0;0];
Area_vector(1)   = Matrix(2,1) * Matrix(3,2) - Matrix(3,1) * Matrix(2,2);
Area_vector(2)   = Matrix(3,1) * Matrix(1,2) - Matrix(1,1) * Matrix(3,2);
Area_vector(3)   = Matrix(1,1) * Matrix(2,2) - Matrix(2,1) * Matrix(1,2);
% Area_vector      = cross(Matrix(:,1),Matrix(:,2));

%--------------------------------------------------------------------------
% Compute area and normal vector
%--------------------------------------------------------------------------
Area             = sqrt(Area_vector' * Area_vector);
Normal           = Area_vector/Area;

end